<?php

declare(strict_types=1);

namespace Drupal\sobki_ui_suite\Service;

use Drupal\Core\Plugin\Discovery\ContainerDerivativeDiscoveryDecorator;
use Drupal\Core\Plugin\Discovery\YamlDiscovery;
use Drupal\ui_styles\StylePluginManager as DefaultStylePluginManager;

/**
 * Overrides the default style plugin manager.
 *
 * @method \Drupal\ui_styles\Definition\StyleDefinition|null getDefinition($plugin_id, $exception_on_invalid = TRUE)
 * @method \Drupal\ui_styles\Definition\StyleDefinition[] getDefinitions()
 *
 * @phpstan-ignore-next-line
 */
class StylePluginManager extends DefaultStylePluginManager {

  /**
   * {@inheritdoc}
   */
  protected function getDiscovery() {
    $this->discovery = new YamlDiscovery('ui_styles', $this->moduleHandler->getModuleDirectories() + $this->themeHandler->getThemeDirectories());
    $this->discovery->addTranslatableProperty('description', 'description_context');
    $this->discovery->addTranslatableProperty('category', 'category_context');
    $this->discovery = new ContainerDerivativeDiscoveryDecorator($this->discovery);
    return $this->discovery;
  }

  /**
   * {@inheritdoc}
   */
  public function alterForm(array $form, array $selected = [], string $extra = '', string $theme = ''): array {
    if (!empty($theme)) {
      $grouped_plugin_definitions = $this->getDefinitionsForTheme($theme);
    }
    else {
      $grouped_plugin_definitions = $this->getGroupedDefinitions();
    }

    if (empty($grouped_plugin_definitions)) {
      return $form;
    }
    $multiple_groups = TRUE;
    if (\count($grouped_plugin_definitions) == 1) {
      $multiple_groups = FALSE;
    }
    $suffix = ' (' . $this->t('used') . ')';
    $global_used = FALSE;
    foreach ($grouped_plugin_definitions as $group_plugin_definitions) {
      $group_used = FALSE;
      $group_key = '';
      foreach ($group_plugin_definitions as $definition) {
        $id = $definition->id();
        $element_name = 'ui_styles_' . $id;
        $plugin_element = [
          '#type' => 'select',
          '#title' => $definition->getLabel(),
          '#description' => $definition->getDescription(),
          '#description_toggle' => TRUE,
          '#options' => $definition->getOptionsAsOptions(),
          '#empty_option' => $this->t('- None -'),
          '#default_value' => $selected[$id] ?? '',
          '#weight' => $definition->getWeight(),
        ];
        // Check if current style is used.
        $used = FALSE;
        if (!empty($plugin_element['#default_value']) && $plugin_element['#default_value'] != $this->t('- None -')) {
          $global_used = TRUE;
          $group_used = TRUE;
          $used = TRUE;
        }

        // Create group if it does not exist yet.
        if ($multiple_groups && $definition->hasCategory()) {
          $group_key = $this->getMachineName($definition->getCategory());
          if (!isset($form[$group_key])) {
            $form[$group_key] = [
              '#type' => 'details',
              '#title' => $definition->getCategory(),
              '#open' => FALSE,
            ];
          }

          $form[$group_key][$element_name] = $plugin_element;
          if ($used) {
            $form[$group_key][$element_name]['#title'] .= $suffix;
          }
        }
        else {
          $form[$element_name] = $plugin_element;
          if ($used) {
            $form[$element_name]['#title'] .= $suffix;
          }
        }
      }

      if (!empty($group_key) && $group_used && isset($form[$group_key]['#title'])) {
        $form[$group_key]['#title'] .= $suffix;
      }
    }
    $form['_ui_styles_extra'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Extra classes'),
      '#description' => $this->t('You can add many values using spaces as separators'),
      '#default_value' => $extra ?: '',
    ];

    if ($global_used && !empty($form['#title'])) {
      $form['#title'] .= $suffix;
    }
    return $form;
  }

}
